@@ -6,7 +6,7 @@ import urllib
 from django import template
 from django.conf import settings
 from django.core.serializers.json import DjangoJSONEncoder
-from django.db.models import DateField, Count
+from django.db.models import DateField, Count, Sum
 from django.db.models.functions import TruncDay
 from django.template.defaultfilters import stringfilter
 from django.urls import translate_url as django_translate_url
@@ -21,13 +21,12 @@ register = template.Library()
 
 @register.simple_tag(takes_context=True)
 def translate_url(context, lang, *args, **kwargs):
-    obj = context.get('object', None)
+    path = kwargs.get('path', None)
+    
+    if not path:
+        obj = context.get('object', None)
+        path = obj.get_absolute_url() if obj else context['request'].path
 
-    if obj:
-        with override_language(lang):
-            return obj.get_absolute_url()
-
-    path = context['request'].path
     return django_translate_url(path, lang)
 
 
@@ -37,8 +36,10 @@ def get_item(value, arg):
 
     numeric_test = re.compile(r'^\d+$')
 
-    if isinstance(value, dict) and arg in value:
+    try:  # for dict and  __getitem__(self, name)
         return value[arg]
+    except (KeyError, TypeError, IndexError):
+        pass
 
     if "." in str(arg):
         firstarg = str(arg).split(".")[0]
@@ -51,11 +52,27 @@ def get_item(value, arg):
         return getattr(value, str(arg))
 
     if numeric_test.match(str(arg)) and len(value) > int(arg):
-        return value[int(arg)]
+        try:
+            return value[int(arg)]
+        except (KeyError, IndexError):
+            pass
+
+    try:
+        dict_value = dict(value)
+
+        if isinstance(dict_value, dict) and arg in dict_value:
+            return dict_value[arg]
+    except ValueError:
+        pass
 
     return getattr(settings, 'TEMPLATE_STRING_IF_INVALID', None)
 
 
+@register.filter()
+def get_list(querydict, param):
+    return querydict.getlist(param)
+
+
 @register.filter
 @stringfilter
 def split(string, sep):
@@ -155,7 +172,18 @@ def filtered_values(filter, request_data):
                                 v_display = str(v_display)
                                 value_values.append(v_display)
                             except (KeyError, AttributeError):
-                                pass
+                                try:
+                                    # grouped choices
+                                    choices = {}
+                                    for group in form.fields[filter_name].choices:
+                                        choices.update(dict(group[1]))
+
+                                    v_display = choices[v]
+                                    v_display = str(v_display)
+                                    value_values.append(v_display)
+                                except (KeyError, AttributeError):
+                                    pass
+
 
                         value = ', '.join(value_values)
                 else:
@@ -226,10 +254,10 @@ def num_applied_filters(filter, request_data):
 def filtered_objects_counts(filtered, all):
     try:
         if filtered == all or all == 0:
-            return mark_safe('%s: <strong>%d</strong>' % (ugettext(u'total'), all))
+            return mark_safe('%s: <strong>%d</strong>' % (ugettext('total'), all))
         else:
             percent = 100 * float(filtered) / all
-            return mark_safe('<strong>%d (%.2f%%)</strong> %s %d' % (filtered, percent, ugettext(u'filtered, from a total of'), all))
+            return mark_safe('<strong>%d (%.2f%%)</strong> %s %d' % (filtered, percent, ugettext('filtered, from a total of'), all))
     except (ValueError, TypeError):
         return ''
 
@@ -267,7 +295,7 @@ def barcode(code, args=None):
     except IllegalCharacterError as e:
         message = unicode(e)
         character = message.split(':', 1)[1]
-        return u'{}: {}'.format(_('Invalid characters'), character)
+        return '{}: {}'.format(_('Invalid characters'), character)
 
 
 @register.inclusion_tag('helpers/pagination.html', takes_context=True)
@@ -606,3 +634,16 @@ def admin_chart(objects, label=_('New data'), color='red', type='bar', date_fiel
         'color': color,
         'type': type
     }
+
+
+@register.simple_tag()
+def objects_stats(objects, count_attr, sum_attr=None):
+    kwargs = {'count': Count(count_attr)}
+
+    if sum_attr:
+        kwargs.update({'sum': Sum(sum_attr)})
+
+    return objects\
+        .values(count_attr)\
+        .annotate(**kwargs)\
+        .order_by('-count')
